/* globals Services, sendAsyncMessage, addMessageListener, removeMessageListener, content */

(function() {
	"use strict";

	var
		app_name = "Viewhance".toLowerCase(),
		app_baseURI = "chrome://" + app_name + "/content/includes/",
		listeners = {},
		_addMessageListener = function(id, fn) {
			_removeMessageListener(id);
			listeners[id] = function(msg) {
				fn(msg.data);
			};
			addMessageListener(id, listeners[id]);
		},
		_removeMessageListener = function(id) {
			if (listeners[id]) {
				removeMessageListener(id, listeners[id]);
			}

			delete listeners[id];
		};

	var observer = {
		unload: function(e) {
			Services.obs.removeObserver(observer, "content-document-global-created");
			observer = listeners = null;
		},
		onDOMReady: function(e) {
			var win = e.target.defaultView;

			if (win.location.protocol === "chrome:"
				&& win.location.host === app_name.toLowerCase()) {
				win.sendAsyncMessage = sendAsyncMessage;
				win.addMessageListener = _addMessageListener;
				win.removeMessageListener = _removeMessageListener;
			}
		},
		observe: function(win) {
			if (!win || win.top !== content) {
				return;
			}

			// baseURI is more reliable
			var location = Services.io.newURI(
				win.location.protocol === "data:" ? "data:text/plain," : win.document.baseURI,
				null,
				null
			);

			if (!(win.document instanceof win.window.ImageDocument
				|| win.document instanceof win.HTMLDocument
					&& (/^(https?|f(tp|ile))$/.test(location.scheme)))) {
				return;
			}

			win = Components.utils.Sandbox([win], {
				sandboxPrototype: win,
				wantComponents: false,
				wantXHRConstructor: false
			});

			win.sendAsyncMessage = sendAsyncMessage;
			win.addMessageListener = _addMessageListener;
			win.removeMessageListener = _removeMessageListener;

			var lss = Services.scriptloader.loadSubScript;

			lss(app_baseURI + "app.js", win);

			if (win.document instanceof win.window.ImageDocument
				|| win.document.querySelector("body>video:only-child:not([src])")) {
				lss(app_baseURI + "viewer.js", win);
			}
			else {
				if (win.document.readyState === "loading") {
					let docReady = function(e) {
						this.removeEventListener(e.type, docReady, true);
						lss(app_baseURI + "opener.js", win);
					};

					win.document.addEventListener("DOMContentLoaded", docReady, true);
				}
				else {
					lss(app_baseURI + "opener.js", win);
				}
			}
		}
	};

	Services.obs.addObserver(observer, "content-document-global-created", false);

	addEventListener("unload", observer.unload, false);

	// for the options page
	addEventListener("DOMContentLoaded", observer.onDOMReady, true);
})();