"use strict";

var browser = {}, app, Port, cfg, _;

if (document instanceof window.HTMLDocument) {
	browser = document.documentElement || document.createElementNS("http://www.w3.org/1999/xhtml", "div");

	if (!window.requestAnimationFrame) {
		window.requestAnimationFrame = window.webkitRequestAnimationFrame
			|| window.mozRequestAnimationFrame || function(cb) { return window.setTimeout(cb, 20); };
	}

	if (browser.style) {
		_ = browser.style;

		browser = {
			"wheel": "onwheel" in browser ? "wheel" : "mousewheel",
			"transform": ("webkitTransform" in _ ? "webkitT" : ("OTransform" in _ ? "OT" : ("MozTransform" in _ ? "MozT" : "t"))) + "ransform",
			"transform_css": ("webkitTransform" in _ ? "-webkit-" : ("OTransform" in _ ? "-o-" : ("MozTransform" in _ ? "-moz-" : ""))) + "transform",
			"transition_css": ("webkitTransition" in _ ? "-webkit-" : ("OTransition" in _ ? "-o-" : ("MozTransition" in _ ? "-moz-" : ""))) + "transition",
			"transitionend": "webkitTransition" in _ ? "webkitTransitionEnd" : ("OTransition" in _ ? "oTransitionEnd" : "transitionend"),
			"box-sizing_css": ("MozBoxSizing" in _ ? "-moz-" : ("WebkitBoxSizing" in _ ? "-webkit-" : "")) + "box-sizing",
			"zoom-in": this.chrome || this.mx || this.safari ? "-webkit-zoom-in" : "zoom-in"
		};
		_ = null;
	}
}

function gen_nodes(host, nodes) {
	if (!host || !Array.isArray(nodes)) {
		return;
	}

	if (nodes.length) {
		var node, attr, i = 0, fragment = document.createDocumentFragment();

		for (; i < nodes.length; ++i) {
			if (nodes[i]) {
				if (typeof nodes[i] === "string") {
					node = document.createTextNode(nodes[i]);
				}
				else {
					node = document.createElement(nodes[i].tag);

					if (nodes[i].attrs) {
						for (attr in nodes[i].attrs) {
							// bypass CSP
							if (attr === "style") {
								node.style.cssText = nodes[i].attrs[attr];
							}
							else {
								node.setAttribute(attr, nodes[i].attrs[attr]);
							}
						}
					}

					if (nodes[i].nodes) {
						gen_nodes(node, nodes[i].nodes);
					}
					else if (nodes[i].text) {
						node.textContent = nodes[i].text;
					}
				}

				fragment.appendChild(node);
			}
		}

		if (fragment.childNodes.length) {
			host.appendChild(fragment);
		}
	}

	return host;
}

var media_type = function() {
	if (document.media_type) {
		return document.media_type;
	}

	var d = document, img = d.images && d.images.length === 1 && d.images[0];

	if (img && img.parentNode === d.body && img.src === window.location.href) {
		return document.media_type = "img";
	}

	d = d.querySelector("body > video:first-child, body > div:only-child > video:first-child");

	if (d && d.currentSrc === window.location.href) {
		return document.media_type = "video";
	}
};

app = {
	"name": /*~APP_NAME~*/"Viewhance",
	"version": /*~APP_VERSION~*/"0.9.6"
};

browser.firefox = true;

Port = {
	listener_id: app.name.toLowerCase() + ":" + parseInt(Math.random() * 1e10, 10).toString(36),
	listen: function(fn) {
		if (typeof fn === "function") {
			addMessageListener(this.listener_id, function(msg) {
				fn(JSON.parse(msg));
			});
		}
		else {
			removeMessageListener(this.listener_id);
		}
	},
	send: function(msg) {
		sendAsyncMessage(app.name.toLowerCase() + ":background", {
			listener_id: this.listener_id,
			data: msg,
			origin: window.location.href
		});
	}
};

if (location.protocol === "chrome:" && location.host === app.name.toLowerCase()) {
	_ = (function() {
		var stringBundle = Components.classes["@mozilla.org/intl/stringbundle;1"]
				.getService(Components.interfaces.nsIStringBundleService)
				.createBundle("chrome://" + app.name + "/locale/options.properties");

		return function(s) {
			try {
				return stringBundle.GetStringFromName(s);
			} catch (ex) {
				return s;
			}
		};
	})();

	browser.insertHTML = (function() {
		var Cc = Components.classes, Ci = Components.interfaces,
			parser = Cc["@mozilla.org/parserutils;1"].getService(Ci.nsIParserUtils),
			io = Cc["@mozilla.org/network/io-service;1"].getService(Ci.nsIIOService);

		return function(node, html) {
			node.appendChild(parser.parseFragment(
				html,
				parser.SanitizerAllowStyle,
				false,
				io.newURI(document.baseURI, null, null),
				document.documentElement
			));
		}
	})();
}
